/*
 * This file is a component of thundr, a software library from 3wks.
 * Read more: http://www.3wks.com.au/thundr
 * Copyright (C) 2013 3wks, <thundr@3wks.com.au>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.threewks.thundr.view.handlebars;

import java.io.IOException;

import javax.servlet.ServletContext;

import com.github.jknack.handlebars.Handlebars;
import com.github.jknack.handlebars.HandlebarsException;
import com.github.jknack.handlebars.Template;
import com.github.jknack.handlebars.cache.HighConcurrencyTemplateCache;
import com.github.jknack.handlebars.helper.StringHelpers;
import com.threewks.thundr.view.ViewResolutionException;

public class HandlebarsRenderer {
	private Handlebars handlebars;

	public HandlebarsRenderer(boolean debug) {
		// debug is no longer used
	}

	public Template getOrCompileTemplate(String templatePath, ServletContext servletContext) {
		init(servletContext);
		try {
			return handlebars.compile(templatePath);
		} catch (IOException e) {
			throw new ViewResolutionException(e, "Failed to find Handlebars template '%s': %s", templatePath, e.getMessage());
		} catch (HandlebarsException e) {
			throw new ViewResolutionException(e, "Failed to compile Handlebars template '%s': %s", templatePath, e.getMessage());
		}
	}

	// package protected for tests
	synchronized void init(ServletContext servletContext) {
		if (this.handlebars == null) {
			this.handlebars = new Handlebars(new CompositeTemplateLoader(servletContext), new HighConcurrencyTemplateCache());
			StringHelpers.register(handlebars);
		}
	}
}