/*
 * This file is a component of thundr, a software library from 3wks.
 * Read more: http://www.3wks.com.au/thundr
 * Copyright (C) 2013 3wks, <thundr@3wks.com.au>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.threewks.thundr.view.handlebars;

import java.io.IOException;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import jodd.util.MimeTypes;

import com.github.jknack.handlebars.Context;
import com.github.jknack.handlebars.Template;
import com.github.jknack.handlebars.context.FieldValueResolver;
import com.github.jknack.handlebars.context.JavaBeanValueResolver;
import com.github.jknack.handlebars.context.MapValueResolver;
import com.github.jknack.handlebars.context.MethodValueResolver;
import com.threewks.thundr.view.ViewResolutionException;
import com.threewks.thundr.view.ViewResolver;

public class HandlebarsViewResolver implements ViewResolver<HandlebarsView> {
	private HandlebarsRenderer handlebarsRenderer;

	public HandlebarsViewResolver(boolean debug) {
		handlebarsRenderer = new HandlebarsRenderer(debug);
	}

	@Override
	public void resolve(HttpServletRequest req, HttpServletResponse resp, HandlebarsView viewResult) {
		String view = viewResult.getView();
		Template template = handlebarsRenderer.getOrCompileTemplate(view, req.getSession().getServletContext());
		try {
			Context context = Context.newBuilder(viewResult.getModel()).resolver(MapValueResolver.INSTANCE,
					JavaBeanValueResolver.INSTANCE,
					FieldValueResolver.INSTANCE,
					MapValueResolver.INSTANCE,
					MethodValueResolver.INSTANCE).build();
			String content = template.apply(context);
			resp.getWriter().write(content);
			resp.setContentType(MimeTypes.MIME_TEXT_HTML);
			resp.setContentLength(content.getBytes().length);
			resp.setStatus(HttpServletResponse.SC_OK);
		} catch (IOException e) {
			throw new ViewResolutionException(e, "Failed to render template '%s': %s", view, e.getMessage());
		}
	}
}
